@@ -55,8 +55,8 @@ function hook_image_effect_info() {
  */
 function hook_image_effect_info_alter(&$effects) {
   // Override the Image module's crop effect with more options.
-  $effect['image_crop']['effect callback'] = 'mymodule_crop_effect';
-  $effect['image_crop']['form callback'] = 'mymodule_crop_form';
+  $effects['image_crop']['effect callback'] = 'mymodule_crop_effect';
+  $effects['image_crop']['form callback'] = 'mymodule_crop_form';
 }
 
 /**
